<?php declare (strict_types = 1);

namespace EuKit\Base\ViewModels;

use EuKit\Base\Parts\Image;
use EuKit\Base\Parts\NewsMLArticle;
use Infomaker\Everyware\NewsML\Objects\TeaserItem;
use Infomaker\Everyware\Support\GenericPropertyObject;
use Infomaker\Everyware\Support\Interfaces\PropertyObject;
use Infomaker\Everyware\Base\OpenContent\QueryBuilder;
use OcAPI as ocapi_clause;
use Everyware\Plugin\SettingsParameters\SettingsParameter;


class Teaser extends BaseObject
{
    /**
     * @var PropertyObject
     */
    private $article;

    /**
     * @var array
     */
    private $args;

    /**
     * @var string|null
     */
    private $imageRatio;

    /**
     * @var string|null
     */
    private $imageSizesData;
    private $relatedCountData;

    public function __construct(PropertyObject $article, array $args = [])
    {
        $this->article = $article;
        $this->args = $args;
    }

    /**
     * @param string|null $imageRatio Examples: '16:9', '3:2'. Null reverts it to the default image ratio.
     */
    public function setImageRatio(?string $imageRatio): void
    {
        $this->imageRatio = $imageRatio;
    }

    /**
     * @param string|null $imageSizesData
     */
    public function setImageSizesData(?string $imageSizesData): void
    {
        $this->imageSizesData = $imageSizesData;
    }

    /**
    * @param string|null $relatedCountData
    */
    public function setRelatedArticleData(string  $boolCountData, string  $relatedCountData): void
    {
        if($boolCountData == 'true')
        {
            $this->relatedCountData = $relatedCountData;
        }
    }

    private function createTeaser(): void
    {
        $propertyObject = new GenericPropertyObject();
        $propertyObject->fill([
            'uuid' => $this->article->getSingleValue('uuid'),
            'pubdate' => $this->article->getPubDate()->diffForHumans(),
            'pubdatetime' => $this->article->getPubDate(),
            'headline' => $this->article->getSingleValue('headline'),
            'leadin' => $this->article->getSingleValue('leadin'),
            'authors' => $this->article->authors,
            'permalink' => $this->article->getSingleValue('permalink'),
            'sectionName' => $this->article->getSingleValue('section'),
            'categoriesName' => $this->article->categories->first() ? $this->article->categories->first()->getFirstOfSingleValue('name') : '',
            'tagsName' => $this->article->tags->first() ? $this->article->tags->first()->getFirstOfSingleValue('name') : '',
            'relatedArticles' => $this->article->articles,
            'cluster_article' => $this->article->getMultiValue('cluster_articles')
        ]);

        

        $teaser = $this->article->getNewsMLTeaser();
        if ($teaser instanceof TeaserItem) {
            $image = $teaser->hasImage() ? new Image($teaser->getImage()) : null;
            $text = $teaser->getItemAttribute('text');

            if ($this->article instanceof NewsMLArticle) {
                $bodyParser = $this->article->getBodyParser();

                if ($image === null) {
                    $imageType = 'x-im/image';
                    $image = $bodyParser->hasType($imageType) ? new Image($bodyParser->firstOfType($imageType)) : null;
                }

                if($text === "") {
                    $leadinType = "preamble";
                    $text = $bodyParser->hasType($leadinType) ? $bodyParser->firstOfType($leadinType)->get('content') : "";
                }
            }

            if ($image instanceof Image) {
                $image->setArgs($this->args);
                if ($this->imageRatio !== null) {
                    $image->setRatio($this->imageRatio);
                }
                if ($this->imageSizesData !== null) {
                    $image->setSizesData($this->imageSizesData);
                }
            }

            $teaserData = array_replace($teaser->toArray(), [
                'image' => $image instanceof Image ? $image->getViewData() : null,
                'text' => $text,
            ]);

            $propertyObject->fill(array_filter($teaserData));

            // --- Related Article for Teasers---------------
            $oc = new ocapi_clause();
            $relatedArticleUuid = [];
            foreach ($propertyObject['cluster_article'] as $uuid) {
                $provider = [
                    "q" => QueryBuilder::where("uuid", $uuid)->buildQueryString(),
                ];
                $relatedArticleUuid[] = $oc->search($provider, true);
            }
            if (isset($relatedArticleUuid) && !empty($relatedArticleUuid)) {
                $relatedArticleArray = array();
                $count = 0;  
                $relatedArticleLimit =0;  
                if ($this->relatedCountData !== null) {
                   $relatedArticleLimit = $this->relatedCountData;
                }

                if($relatedArticleLimit > 0){
                    foreach ($relatedArticleUuid as $relatedArticleValue) {  
                        if(isset($relatedArticleLimit) && (int)$relatedArticleLimit>0){
                            if ($count == $relatedArticleLimit) {
                                break;
                            }
                        }                    
                        
                        $result = $oc->get_single_article(
                            $relatedArticleValue[0]["prop_data"]["uuid"][0]
                        );

                        $article_child = $result["article"] ?: null;
                        if ($article_child instanceof \OcArticle) {
                            $relatedArticleArray[$count][
                                "permalink"
                            ] = $article_child->get_permalink();
                        } else {
                            $relatedArticleArray[$count]["permalink"] = "";
                        } 


                        $relatedArticleArray[$count]["headline"] =
                            $relatedArticleValue[0]["prop_data"]["headline"][0];
                        $relatedArticleArray[$count]["body_raw"] = strip_tags(
                            $relatedArticleValue[0]["prop_data"]["bodyraw"][0]
                        );
                        $relatedArticleArray[$count]["pubdate"] = date(
                            "D j M Y",
                            strtotime($relatedArticleValue[0]["prop_data"]["pubdate"][0])
                        );
                        ++$count;
                    }
                }

                $propertyObject->fill(array("cluster_array" => $relatedArticleArray));
            }

            
        }
        parent::__construct($propertyObject);
    }

    public function getViewData(): array {
        if ($this->viewData === null) {
            $this->createTeaser();
        }

        return parent::getViewData();
    }
}
